// Замініть на адресу вашого контракту
const contractAddress = "0xca0c2D17D2E5C9761F58799f4fEC3DF2C8b57EF8";

// Змінна для зберігання ABI - буде отримана з метаданих
let contractABI;

// DOM Елементи
const connectButton = document.getElementById("connectButton");
const accountAddress = document.getElementById("accountAddress");
const balanceDisplay = document.getElementById("balance");
const transferForm = document.getElementById("transferForm");
const transferStatus = document.getElementById("transferStatus");
const formErrors = document.getElementById("formErrors");

const tokenNameEl = document.getElementById("tokenName");
const tokenSymbolEl = document.getElementById("tokenSymbol");
const tokenDecimalsEl = document.getElementById("tokenDecimals");
const tokenTotalSupplyEl = document.getElementById("tokenTotalSupply");

// Змінні Ethers.js
let provider;
let signer;
let contract;

// Функція для отримання ABI з JSON файлу або ендпоінту
async function fetchABI() {
  try {
    const response = await fetch("SimpleToken.json"); // Переконайтеся, що шлях правильний
    if (!response.ok) {
      throw new Error(`HTTP помилка! Статус: ${response.status}`);
    }
    const data = await response.json();
    contractABI = data;
  } catch (error) {
    console.error("Помилка при отриманні ABI:", error);
    alert("Не вдалося завантажити ABI контракту. Перевірте консоль для деталей.");
  }
}

// Функція для ініціалізації DApp
async function init() {
  await fetchABI();

  if (typeof window.ethereum === "undefined") {
    alert("MetaMask не встановлено. Будь ласка, встановіть MetaMask для використання цього DApp.");
    return;
  }

  provider = new ethers.providers.Web3Provider(window.ethereum);

  // Обробник події для кнопки Підключити Гаманець
  connectButton.addEventListener("click", connectWallet);

  // Обробник події для відправки форми
  transferForm.addEventListener("submit", handleTransfer);

  // Слухач для змін акаунтів
  window.ethereum.on("accountsChanged", handleAccountsChanged);

  // Ініціалізація екземпляру контракту
  contract = new ethers.Contract(contractAddress, contractABI, provider);

  // Отримання та відображення інформації про токен
  await displayTokenInfo();
}

// Функція для обробки підключення гаманця
async function connectWallet() {
  try {
    await provider.send("eth_requestAccounts", []);
    signer = provider.getSigner();
    const address = await signer.getAddress();
    accountAddress.textContent = `Підключено: ${address}`;
    // Оновлення контракту з підписувачем для операцій запису
    contract = contract.connect(signer);
    // Отримання та відображення балансу користувача
    await displayUserBalance(address);
  } catch (error) {
    console.error("Помилка при підключенні гаманця:", error);
    alert("Не вдалося підключити гаманець. Перевірте консоль для деталей.");
  }
}

// Функція для обробки змін акаунтів
async function handleAccountsChanged(accounts) {
  if (accounts.length === 0) {
    accountAddress.textContent = "Не підключено";
    balanceDisplay.textContent = "-";
  } else {
    const address = accounts[0];
    accountAddress.textContent = `Підключено: ${address}`;
    await displayUserBalance(address);
  }
}

// Функція для отримання та відображення інформації про токен
async function displayTokenInfo() {
  try {
    const name = await contract.name();
    const symbol = await contract.symbol();
    const decimals = await contract.decimals();
    const totalSupply = await contract.totalSupply();

    tokenNameEl.textContent = name;
    tokenSymbolEl.textContent = symbol;
    tokenDecimalsEl.textContent = decimals;
    tokenTotalSupplyEl.textContent = ethers.utils.formatUnits(totalSupply, decimals);
  } catch (error) {
    console.error("Помилка при отриманні інформації про токен:", error);
    alert("Не вдалося отримати інформацію про токен. Перевірте консоль для деталей.");
  }
}

// Функція для отримання та відображення балансу користувача
async function displayUserBalance(address) {
  try {
    const balance = await contract.balanceOf(address);
    const decimals = await contract.decimals();
    balanceDisplay.textContent = `${ethers.utils.formatUnits(balance, decimals)} ${await contract.symbol()}`;
  } catch (error) {
    console.error("Помилка при отриманні балансу:", error);
    balanceDisplay.textContent = "Помилка при отриманні балансу";
  }
}

// Функція для обробки передачі токенів
async function handleTransfer(event) {
  event.preventDefault();
  formErrors.textContent = "";
  transferStatus.textContent = "";

  const recipient = document.getElementById("recipient").value.trim();
  const amount = document.getElementById("amount").value.trim();

  // Валідація Введених Даних
  const errors = [];

  if (!ethers.utils.isAddress(recipient)) {
    errors.push("Невірна адреса отримувача.");
  }

  if (isNaN(amount) || parseFloat(amount) <= 0) {
    errors.push("Кількість повинна бути позитивним числом.");
  }

  if (errors.length > 0) {
    formErrors.textContent = errors.join(" ");
    return;
  }

  try {
    // Парсинг кількості на основі десяткових
    const decimals = await contract.decimals();
    const parsedAmount = ethers.utils.parseUnits(amount, decimals);

    // Перевірка балансу користувача
    const userBalance = await contract.balanceOf(await signer.getAddress());
    if (parsedAmount.gt(userBalance)) {
      formErrors.textContent = "Недостатній баланс.";
      return;
    }

    // Відправка транзакції передачі
    const tx = await contract.transfer(recipient, parsedAmount);
    transferStatus.textContent = "Транзакція відправлена. Очікування підтвердження...";

    await tx.wait();
    transferStatus.textContent = "Передача успішна!";

    // Оновлення балансу користувача
    const address = await signer.getAddress();
    await displayUserBalance(address);

    // Скидання форми
    transferForm.reset();
  } catch (error) {
    console.error("Помилка під час передачі:", error);
    if (error.code === 4001) {
      // Користувач відхилив транзакцію
      transferStatus.textContent = "Транзакція відхилена користувачем.";
    } else {
      transferStatus.textContent = "Передача не вдалася. Перевірте консоль для деталей.";
    }
  }
}

// Ініціалізація DApp при завантаженні сторінки
window.addEventListener("load", init);
